@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: i18n.h,v 1.1 2002/03/01 00:23:03 f1rmb Exp $
+ * $Id: i18n.h,v 1.2 2004/11/10 01:29:12 dsalt Exp $
  *
  * intl macros
  */
@@ -39,6 +39,13 @@
 #    define dgettext(Domain,Message) (Message)
 #    define dcgettext(Domain,Message,Type) (Message)
 #    define bindtextdomain(Domain,Directory) (Domain)
+#    define ngettext(Singular,Plural,N) \
+	    ((N) == 1 ? (Singular) : (Plural))
+#    define dngettext(Domain,Singular,Plural,N) \
+	    ((N) == 1 ? (Singular) : (Plural))
+#    define dncgettext(Domain,Singular,Plural,N,Type) \
+	    ((N) == 1 ? (Singular) : (Plural))
+#    define bind_textdomain_codeset(Domain,Codeset) ((Codeset))
 #    define _(String) (String)
 #    define N_(String) (String)
 #endif
