@@ -1,3 +1,5 @@
+#include "config.h"
+#include "i18n.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -7,10 +9,19 @@
 #include <sys/un.h>
 #include <string.h>
 #include <fcntl.h>
+#include <signal.h>
 
 #define SOCKET_FILENAME "%s/.gxine/socket"
 #define BUF_LEN 1024
 
+static volatile int done = 0;
+
+static void sigpipe (int sig)
+{
+  done = 1;
+  signal (SIGPIPE, sigpipe);
+}
+
 int main (int argc, char **argv) {
 
   int                fd;
@@ -18,11 +29,14 @@ int main (int argc, char **argv) {
   int                length = sizeof(struct sockaddr_un);
   char               filename [1024];
   char              *tstr;
+  int		     ret = 0;
+
+  signal (SIGPIPE, sigpipe);
 
   /* server filename */
   snprintf (filename, 1024, SOCKET_FILENAME, getenv ("HOME")); 
 
-  printf ("connecting to %s...\n", filename);
+  printf (_("Connecting to %s...\n"), filename);
 
   fd = socket (AF_UNIX, SOCK_STREAM, 0);
 
@@ -50,18 +64,18 @@ int main (int argc, char **argv) {
     exit (EXIT_FAILURE);
   }
   
-  printf ("connected.\n");
+  printf (_("Connected.\n"));
 
   if (argc>1) {
 
     /* noninteractive */
 
-    printf ("sending command\n");
+    printf (_("Sending command\n"));
 
     write (fd, argv[1], strlen(argv[1]));
     write (fd, "\n", 1);
 
-    printf ("done.\n");
+    printf (_("Done.\n"));
 
   } else {
     /* interactive mode */
@@ -72,7 +86,7 @@ int main (int argc, char **argv) {
     else
       n=STDIN_FILENO+1;
 
-    while (1) {
+    while (!done) {
 
       char buf[10];
       fd_set rset;
@@ -83,7 +97,8 @@ int main (int argc, char **argv) {
 
       if (select (n, &rset, NULL, NULL, NULL) <= 0) {
 	perror ("client: select: ");
-	return -1;
+	ret = 2;
+	break;
       }
 
       if (FD_ISSET (fd, &rset)) {
@@ -102,5 +117,5 @@ int main (int argc, char **argv) {
 
   unlink (tstr);
 
-  return 0;
+  return ret;
 }
