@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 the xine-project
+ * Copyright (C) 2002-2004 the xine-project
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: infobar.h,v 1.1 2002/10/20 14:06:35 guenter Exp $
+ * $Id: infobar.h,v 1.3 2004/11/26 22:57:01 dsalt Exp $
  *
  * a nice black information display area
  */
@@ -31,7 +31,10 @@ infobar_t *create_infobar (GtkWidget **w);
 
 void postinit_infobar (GtkWidget *window, infobar_t *bar) ;
 
-void infobar_line1 (infobar_t *bar, const char *format, ...);
-void infobar_line2 (infobar_t *bar, const char *format, ...);
+void infobar_show_metadata (void);
+
+void infobar_clear (infobar_t *bar);
+void infobar_line (infobar_t *bar, unsigned int lineno,
+		   const char *format, ...);
 
 #endif
