@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: key_events.h,v 1.2 2003/02/22 20:29:48 guenter Exp $
+ * $Id: key_events.h,v 1.4 2004/12/15 23:59:59 dsalt Exp $
  *
  * key event handler, keymap editor
  */
@@ -26,7 +26,21 @@
 
 #include <gtk/gtk.h>
 
-gint key_cb (GtkWidget *win, GdkEventKey *event, gpointer w) ;
+struct kb_xine_event_id_s
+{
+  char name[16];
+  int number;
+};
+typedef struct kb_xine_event_id_s kb_xine_event_id_t;
+
+struct kb_xine_event_map_s {
+  char prefix[8];
+  const kb_xine_event_id_t id[];
+};
+typedef struct kb_xine_event_map_s kb_xine_event_map_t;
+
+gint keypress_cb (GtkWidget *win, GdkEventKey *event);
+gint keyrelease_cb (GtkWidget *win, GdkEventKey *event);
 
 void kb_edit_show (void) ;
 
@@ -34,4 +48,7 @@ void key_events_init (void) ;
 
 void save_key_bindings (void) ;
 
+/* returns xine event number (XINE_EVENT_*) or 0 on failure */
+int kb_xine_event_lookup (const kb_xine_event_map_t *, const char *);
+
 #endif
