@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2003 the xine-project
+ * Copyright (C) 2001-2004 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,14 +16,15 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: main.c,v 1.85 2003/03/23 23:29:30 guenter Exp $
+ * $Id: main.c,v 1.104 2004/12/14 23:52:37 dsalt Exp $
  *
- * gtk+ ui for xine
+ * gtk2 ui for xine
  *
  * main
  */
 
-#include <config.h>
+#include "globals.h"
+
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -40,8 +41,8 @@
 #include <gdk/gdkkeysyms.h>
 #include <glib.h>
 
-#include "globals.h"
-#include "actions.h"
+#include "engine.h"
+#include "player.h"
 #include "infobar.h"
 #include "wizards.h"
 #include "skin_window.h"
@@ -54,24 +55,22 @@
 #include "mediamarks.h"
 #include "playlist.h"
 #include "settings.h"
+#include "snapshot.h"
 #include "play_item.h"
 #include "lirc.h"
 #include "stream_info.h"
 #include "utils.h"
-#include "gtkxine.h"
-
-/*
-#define EXP_STUFF 
-*/
 
 /*
  * globals
  */
 
-GtkWidget      *app;
-GtkWidget      *gtx;
-infobar_t      *bar;
-pthread_mutex_t engine_lock;
+int             verbosity;
+GtkWidget      *app=NULL;
+infobar_t      *bar=NULL;
+gchar          *plugindir, *bindir, *logodir, *pixmapdir, *miscdir;
+gchar          *video_driver_id=NULL;
+gchar          *audio_driver_id=NULL;
 
 void gxine_try_remote (int argc, char *argv[], int enqueue) {
 
@@ -120,14 +119,12 @@ void gxine_try_remote (int argc, char *argv[], int enqueue) {
       server_client_send ("playlist_add (\"");
       server_client_send (mrl);
       server_client_send ("\");\n");
-      i++;
     }
 
     if (!enqueue)
       server_client_send ("playlist_play (0)\n");
   }
 
-  server_client_stop ();
   exit (0);
 }
 
@@ -142,11 +139,14 @@ static void splash_show (void) {
 
   splash = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
-  gtk_window_set_decorated (splash, FALSE);
+  gtk_window_set_decorated (GTK_WINDOW (splash), FALSE);
 
-  gtk_window_set_position (splash, GTK_WIN_POS_CENTER_ALWAYS);
+  gtk_window_set_type_hint (GTK_WINDOW (splash),
+			    GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);
+  gtk_window_set_position (GTK_WINDOW (splash), GTK_WIN_POS_CENTER_ALWAYS);
+  gtk_window_set_keep_above (GTK_WINDOW (splash), TRUE);
 
-  pathname = g_strconcat (GXINE_PIXMAPDIR, "/splash.png", NULL);
+  pathname = g_strconcat (pixmapdir, "/splash.png", NULL);
   img = gtk_image_new_from_file (pathname);
   g_free (pathname);
 
@@ -160,36 +160,76 @@ static void splash_show (void) {
   gdk_threads_leave();  
 }
 
-void splash_hide (void) {
-  gdk_threads_enter();  
-  gtk_widget_destroy (splash);
-  gdk_threads_leave();  
+#ifndef GLIB_USES_SYSTEM_MALLOC
+#include <assert.h>
+gpointer xmalloc (gsize size)
+{
+  gpointer p = malloc (size);
+  if (size)
+    assert (p != NULL);
+  return p;
+}
+
+gpointer xrealloc (gpointer p, gsize size)
+{
+  p = realloc (p, size);
+  if (size)
+    assert (p != NULL);
+  return p;
+}
+#endif
+
+/* Initialise some engine settings from saved preferences */
+static gboolean post_init_configure (void)
+{
+  xine_cfg_entry_t entry;
+
+  if (xine_config_lookup_entry (xine, "gui.deinterlace_by_default", &entry))
+    xine_set_param (stream, XINE_PARAM_VO_DEINTERLACE, entry.num_value);
+  return FALSE;
 }
 
 int main(int argc, char* argv[]) {
 
   int            opt, enqueue;
 
-  gchar         *video_driver_id=NULL;
-  gchar         *audio_driver_id=NULL;
+#ifndef GLIB_USES_SYSTEM_MALLOC
+  GMemVTable vtab = { xmalloc, xrealloc, free, NULL, NULL, NULL };
+  g_mem_set_vtable (&vtab);
+#endif
 
   /*
    * init glib/gdk/gtk thread safe/aware 
    */
 
+#ifdef ENABLE_NLS
+  setlocale (LC_ALL, "");
+#endif
+
   g_thread_init (NULL);
   gdk_threads_init ();
   gtk_init(&argc, &argv);
 
+#ifdef ENABLE_NLS
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+#endif
+
   /*
-   * some xine engine functions may block for a long
-   * time - that's why it is often not called directly
-   * but by using an extra thread so the gui will stay
-   * responsive
+   * init paths here. defaults are compiled in and may
+   * be overwritten by environment variables
    */
 
-  pthread_mutex_init (&engine_lock, NULL);
-
+  plugindir = getenv("GXINE_PLUGINDIR");
+  if (!plugindir) plugindir = g_strconcat(GXINE_PLUGINDIR, NULL);
+  bindir = getenv("GXINE_BINDIR");
+  if (!bindir) bindir = g_strconcat(GXINE_BINDIR, NULL);
+  logodir = getenv("GXINE_LOGODIR");
+  if (!logodir) logodir = g_strconcat(GXINE_LOGODIR, NULL);
+  pixmapdir = getenv("GXINE_PIXMAPDIR");
+  if (!pixmapdir) pixmapdir = g_strconcat(GXINE_PIXMAPDIR, NULL);
+  miscdir = getenv("GXINE_MISCDIR");
+  if (!miscdir) miscdir = g_strconcat(GXINE_MISCDIR, NULL);
 
   /*
    * parse command line arguments
@@ -212,7 +252,7 @@ int main(int argc, char* argv[]) {
       audio_driver_id = optarg;
       break;  
     default:
-      printf ("usage: %s [-v] [-e] [-V <video_driver>] [-A <audio_driver>] mrls...\n\n", 
+      printf (_("usage: %s [-v] [-e] [-V <video_driver>] [-A <audio_driver>] mrls...\n\n"), 
 	      argv[0]);
       exit (1);
     }
@@ -242,42 +282,51 @@ int main(int argc, char* argv[]) {
 
   splash_show ();
 
-
   /*
-   * skript engine, main window
+   * init xine, set up skript engine, main window
    */
 
-  action_init ();
+  gdk_threads_enter ();
+
+  gtk_window_set_default_icon (load_icon ("gxine-logo.png"));
+
+  engine_init ();
+  player_init ();
 
 #ifdef EXP_STUFF
   create_skin_window () ;
+#else
+  noskin_main_init ();
 #endif
-  noskin_main_init (audio_driver_id, video_driver_id);
 
   /*
    * create all dialogs (invisible)
    */
 
   file_dialog_init ();
+  utils_init       ();
   open_mrl_init    ();
   log_window_init  ();
   preferences_init ();
   playlist_init    ();
   settings_init    ();
-  mediamarks_init  (); 
+  mediamarks_init  ();
   play_item_init   ();
   key_events_init  ();
   gxine_lirc_init  ();
-  utils_init       ();
+  snapshot_init    ();
+  ui_init	   ();
   stream_info_init ();
-  
+  wizards_init     ();
+
+  gtk_widget_destroy (splash);
+
   /*
    * wizards (first run only)
    */
 
-  gdk_threads_enter ();  
   run_wizards (FALSE);
-  gdk_threads_leave ();  
+  gdk_threads_leave ();
 
   /*
    * argument parsing
@@ -293,9 +342,9 @@ int main(int argc, char* argv[]) {
     for (i=optind; i<argc; i++) {
 
       if (first<0) {
-	first = playlist_add_mrl (argv[i]);
+	first = playlist_add_mrl (argv[i], -1);
       } else {
-	playlist_add_mrl (argv[i]);
+	playlist_add_mrl (argv[i], -1);
       }
     }
 
@@ -304,12 +353,11 @@ int main(int argc, char* argv[]) {
     playlist_logo ();
 
   server_start ();
-  
-  splash_hide (); 
+  g_idle_add ((GSourceFunc) post_init_configure, NULL);
 
-  gdk_threads_enter();  
+  gdk_threads_enter();
   gtk_main();
-  gdk_threads_leave();  
+  gdk_threads_leave();
 
   return 0;
 }
