@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2003 the xine-project
+ * Copyright (C) 2002-2004 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: mediamarks.c,v 1.18 2003/03/08 15:50:36 guenter Exp $
+ * $Id: mediamarks.c,v 1.37 2004/12/17 00:33:00 dsalt Exp $
  *
  * mediamarks - load
  *            - save
@@ -24,29 +24,27 @@
  * functions
  */
 
-#include "config.h"
+#include "globals.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <glib.h>
 
-#include "globals.h"
 #include "utils.h"
-#include "gtkxine.h"
-#include "actions.h"
+#include "engine.h"
 #include "mediamarks.h"
+#include "menu.h"
 #include "playlist.h"
 #include "play_item.h"
 
-#define PLACEHOLDER "(---)"
-
 static GtkTreeStore *mm_store;
-static GtkWidget    *manage_dlg, *cat_dlg, *cat_entry;
+static GtkWidget    *manage_dlg, *cat_dlg, *cat_entry, *mm_tree;
 static GtkTreeIter   cat_iter;
-static int           is_visible;
+static gboolean      is_visible, cat_edited;
 
 static void menu_cb (GtkWidget* widget, gpointer data) {
 
@@ -58,7 +56,7 @@ static void menu_cb (GtkWidget* widget, gpointer data) {
 
     item = play_item_copy (item_orig);
 
-    pos = playlist_add (item);
+    pos = playlist_add (item, -1);
     playlist_play (pos);
   }
 }
@@ -81,7 +79,7 @@ static void gen_items (GtkMenuShell *submenu, GtkTreeIter *iter) {
 			      iter, 0, &v);
     id = g_value_peek_pointer (&v);
 
-    if (id && strcmp (id, PLACEHOLDER)) {
+    if (id) {
       item = gtk_menu_item_new_with_label (id);
       g_value_unset (&v);
 
@@ -126,6 +124,9 @@ static void update_menu (GtkMenuShell *submenu) {
   GtkWidget    *item;
   GtkTreeIter   iter;
 
+  if (!submenu)
+    return;
+
   /*
    * remove old items (if any)
    */
@@ -142,8 +143,6 @@ static void update_menu (GtkMenuShell *submenu) {
 
   if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (mm_store), &iter))
     gen_items (submenu, &iter);
-
-
 }
 
 /* 
@@ -151,23 +150,19 @@ static void update_menu (GtkMenuShell *submenu) {
  */
 
 static void get_items (xml_node_t *node, int depth, GtkTreeIter *parent) {
+  /* assumption: *one* of (depth == 0) or (parent == NULL) */
 
   while (node) {
 
     if (!strcasecmp (node->name, "sub")) {
 
-      GtkTreeIter iter, placeholder;
+      GtkTreeIter iter;
       char       *id;
 
       id = xml_parser_get_property (node, "name");
 
-      if (depth)
-	gtk_tree_store_append (mm_store, &iter, parent);
-      else
-	gtk_tree_store_append (mm_store, &iter, NULL);
+      gtk_tree_store_append (mm_store, &iter, parent);
       gtk_tree_store_set (mm_store, &iter, 0, id, -1);
-      gtk_tree_store_append (mm_store, &placeholder, &iter);
-      gtk_tree_store_set (mm_store, &placeholder, 0, PLACEHOLDER, -1);
 
       get_items (node->child, depth+1, &iter);
 
@@ -175,15 +170,12 @@ static void get_items (xml_node_t *node, int depth, GtkTreeIter *parent) {
       play_item_t *play_item = play_item_load (node->child);
       GtkTreeIter  iter;
 
-      if (depth)
-	gtk_tree_store_append (mm_store, &iter, parent);
-      else
-	gtk_tree_store_append (mm_store, &iter, NULL);
+      gtk_tree_store_append (mm_store, &iter, parent);
       gtk_tree_store_set (mm_store, &iter, 0, play_item->title, 1, 
 			  play_item->mrl, 2, play_item, -1);
 
     } else {
-      printf ("mediamarks: error, unknown node type %s\n", node->name);
+      printf (_("mediamarks: error, unknown node type %s\n"), node->name);
     }
 
     node = node->next;
@@ -196,7 +188,7 @@ static void load_new_file (xml_node_t *node) {
 
     get_items (node->child, 0, NULL);
   } else {
-    printf ("mediamarks: ERROR, root node must be GXINEMM\n");
+    printf (_("mediamarks: ERROR, root node must be GXINEMM\n"));
   }
 }
 
@@ -236,7 +228,6 @@ static void load_old_file (char *fname) {
     depth = 0;
 
     while (1) {
-      
       char *id, *str;
 
       id = read_line (f);
@@ -245,18 +236,10 @@ static void load_old_file (char *fname) {
 
       if (!strncmp (id, "<sub>", 5)) {
 
-	GtkTreeIter placeholder;
-	/* sub-tree starts here */
-
-	if (depth)
-	  gtk_tree_store_append (mm_store, &iter, &parent);
-	else
-	  gtk_tree_store_append (mm_store, &iter, NULL);
+	gtk_tree_store_append (mm_store, &iter, depth ? &parent : NULL);
 	gtk_tree_store_set (mm_store, &iter, 0, &id[5], -1);
-	gtk_tree_store_append (mm_store, &placeholder, &iter);
-	gtk_tree_store_set (mm_store, &placeholder, 0, PLACEHOLDER, -1);
 
-	memcpy (&parent, &iter, sizeof (iter));
+	parent = iter;
 	depth++;
 
       } else if (!strncmp (id, "</sub>", 6)) {
@@ -268,17 +251,17 @@ static void load_old_file (char *fname) {
 	depth--;
 	if (depth<0) {
 
-	  printf ("mediamarks: mediamarks file %s corrupted, ignoring rest of file\n",
+	  printf (_("mediamarks: media marks file %s corrupted - ignoring rest of file\n"),
 		  fname);
 	  return;
 	}
-	memcpy (&parent, &iter, sizeof (iter));
+	parent = iter;
 
       } else { 
 	
 	play_item_t *play_item;
 	char        *mrl;
-	int          time;
+	int          time = 0;
 
 	mrl = read_line (f);
 	if (!mrl)
@@ -291,10 +274,7 @@ static void load_old_file (char *fname) {
 
 	play_item = play_item_new (id, mrl, time);
 
-	if (depth)
-	  gtk_tree_store_append (mm_store, &iter, &parent);
-	else
-	  gtk_tree_store_append (mm_store, &iter, NULL);
+	gtk_tree_store_append (mm_store, &iter, depth ? &parent : NULL);
 	gtk_tree_store_set (mm_store, &iter, 0, id, 1, mrl, 2, play_item, -1);
       }
 
@@ -306,42 +286,39 @@ static void load_old_file (char *fname) {
     fclose (f);
   } else {
 
-    printf ("mediamarks: load failed!\n");
+    printf (_("mediamarks: load failed!\n"));
 
   }
 }
 
-static void win_delete_cb (GtkWidget* widget, gpointer data) {
-  gtk_widget_hide (widget);
-  is_visible = 0;
+static void cat_response_cb (GtkDialog* widget, int response, gpointer data)
+{
+  switch (response)
+  {
+  case GTK_RESPONSE_OK:
+    cat_edited = TRUE;
+  default:
+    gtk_widget_hide (cat_dlg);
+    gtk_main_quit();
+  }
 }
 
-static gboolean close_cb (GtkWidget* widget, gpointer data) {
-  gtk_widget_hide (manage_dlg);
-  is_visible = 0;
+static gboolean do_category_edit (void)
+{
+  cat_edited = FALSE;
+  gtk_widget_show_all (cat_dlg);
+  if (!GTK_WINDOW(cat_dlg)->modal)
+    gtk_window_set_modal (GTK_WINDOW(cat_dlg), TRUE);
+  gtk_widget_grab_focus (cat_entry);
 
-  return TRUE;
-}
+  gtk_main();
 
-static void cat_apply (void) {
-  const char *name = gtk_entry_get_text (GTK_ENTRY(cat_entry));
-
-  gtk_tree_store_set (mm_store, &cat_iter, 0, name, -1);
-}
-
-static void cat_delete_cb (GtkWidget* widget, gpointer data) {
-  gtk_widget_hide (widget);
-  cat_apply ();
-}
-
-static void cat_close_cb (GtkWidget* widget, gpointer data) {
-  gtk_widget_hide (data);
-  cat_apply ();
+  return cat_edited;
 }
 
 static void edit_cb (GtkWidget* widget, gpointer data) {
 
-  GtkTreeView       *tree = (GtkTreeView *) data;
+  GtkTreeView       *tree = GTK_TREE_VIEW (mm_tree);
   GtkTreeSelection  *sel;
   GtkTreeIter        iter;  
 
@@ -356,22 +333,15 @@ static void edit_cb (GtkWidget* widget, gpointer data) {
 			      &iter, 2, &v);
     play_item = g_value_peek_pointer (&v);
     if (play_item) {
-#ifdef LOG
-      printf ("mediamarks: got a play item\n");
-#endif
-      
-      play_item_edit (play_item);
-
-#ifdef LOG
-      printf ("mediamarks: play item title after edit : %s\n",
-	      play_item->title);
-#endif
-
-      gtk_tree_store_set (mm_store, &iter, 
-			  0, play_item->title, 
-			  1, play_item->mrl, 2, play_item, -1);
-
-
+      logprintf ("mediamarks: got a play item\n");
+      if (play_item_edit (play_item, PLAY_ITEM_MEDIAMARK, NULL))
+      {
+	logprintf ("mediamarks: play item title after edit : %s\n",
+		   play_item->title);
+	gtk_tree_store_set (mm_store, &iter,
+			    0, play_item->title,
+			    1, play_item->mrl, 2, play_item, -1);
+      }
     } else {
       GValue       v;
       char        *name;
@@ -385,9 +355,9 @@ static void edit_cb (GtkWidget* widget, gpointer data) {
       gtk_entry_set_text (GTK_ENTRY (cat_entry), name);
       g_value_unset (&v);
 
-      gtk_widget_show_all (cat_dlg);
-      gtk_widget_map (cat_dlg);
-      gtk_widget_grab_focus (cat_entry);
+      if (do_category_edit ())
+	gtk_tree_store_set (mm_store, &cat_iter, 0,
+			    gtk_entry_get_text (GTK_ENTRY(cat_entry)), -1);
     }
     g_value_unset (&v);
   }
@@ -395,57 +365,33 @@ static void edit_cb (GtkWidget* widget, gpointer data) {
 
 static void delete_cb (GtkWidget* widget, gpointer data) {
 
-  GtkTreeView       *tree = (GtkTreeView *) data;
+  GtkTreeView       *tree = GTK_TREE_VIEW (mm_tree);
   GtkTreeSelection  *sel;
   GtkTreeIter        iter;  
 
   sel = gtk_tree_view_get_selection (tree);
 
-  if (gtk_tree_selection_get_selected (sel, NULL, &iter)) {
-    GValue       v;
-    char        *id;
-
-    memset (&v, 0, sizeof (GValue));
-
-    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
-			      &iter, 0, &v);
-    id = (char *) g_value_peek_pointer (&v);
-
-    if (!strcmp (id, PLACEHOLDER)) {
-      g_value_unset (&v);
-      return;
-    }
-
-    g_value_unset (&v);
-
+  if (gtk_tree_selection_get_selected (sel, NULL, &iter))
     gtk_tree_store_remove (mm_store, &iter);
-  }
 }
 
 static void new_category_cb (GtkWidget* widget, gpointer data) {
 
-  GtkTreeView       *tree = (GtkTreeView *) data;
+  GtkTreeView       *tree = GTK_TREE_VIEW (mm_tree);
   GtkTreeSelection  *sel;
-  GtkTreeIter        iter, parent;  
-  int                in_root;
+  GtkTreeIter        iter, parent;   
+  gboolean           in_root = TRUE;
 
   /*
    * find out where to create the new section
    */
 
-  in_root = 1;
   sel     = gtk_tree_view_get_selection (tree);
 
   if (gtk_tree_selection_get_selected (sel, NULL, &iter)) {
     GValue       v;
     play_item_t *play_item;
 
-    memset (&v, 0, sizeof (GValue));
-    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
-			      &iter, 0, &v);
-
-    g_value_unset (&v);
-
     /*
      * is this a section or a play item ?
      */
@@ -455,72 +401,50 @@ static void new_category_cb (GtkWidget* widget, gpointer data) {
 			      &iter, 2, &v);
     play_item = g_value_peek_pointer (&v);
     if (play_item) {
-      
-      if (!gtk_tree_model_iter_parent (GTK_TREE_MODEL (mm_store), 
-				       &parent, &iter)) {
-	in_root = 1;
-      } else
-	in_root = 0;
+      in_root = !gtk_tree_model_iter_parent (GTK_TREE_MODEL (mm_store),
+					     &parent, &iter);
     } else {
       GtkTreePath *path;
-
       path = gtk_tree_model_get_path (GTK_TREE_MODEL (mm_store), &iter);
-      
-      if (gtk_tree_view_row_expanded (tree, path)) {
 
+      if (gtk_tree_view_row_expanded (tree, path)) {
 	in_root = 0;
 	memcpy (&parent, &iter, sizeof (iter));
-
       } else {
-
-	if (!gtk_tree_model_iter_parent (GTK_TREE_MODEL (mm_store), 
-					 &parent, &iter)) {
-	  in_root = 1;
-	} else
-	  in_root = 0;
+	in_root = !gtk_tree_model_iter_parent (GTK_TREE_MODEL (mm_store),
+					       &parent, &iter);
       }
-      
+
       gtk_tree_path_free (path);
     }
     g_value_unset (&v);
-  } else {
-    in_root = 1;
   }
 
-  /*
-   * create new category
-   */
-
-  if (in_root)
-    gtk_tree_store_append (mm_store, &cat_iter, NULL);
-  else
-    gtk_tree_store_append (mm_store, &cat_iter, &parent);
-  gtk_tree_store_set (mm_store, &cat_iter, 0, "New Category", -1);
-  /* create dummy entry so d&d works */
-  gtk_tree_store_append (mm_store, &iter, &cat_iter);
-  gtk_tree_store_set (mm_store, &iter, 0, PLACEHOLDER, -1);
-  
   /*
    * show dialog, ask for section name
    */
 
-  gtk_entry_set_text (GTK_ENTRY (cat_entry), "New Category");
-  gtk_widget_show_all (cat_dlg);
-  gtk_widget_map (cat_dlg);
-  gtk_widget_grab_focus (cat_entry);
+  gtk_entry_set_text (GTK_ENTRY (cat_entry), _("New Category"));
+
+  if (do_category_edit ())
+  {
+    /* create new category */
+    gtk_tree_store_append (mm_store, &cat_iter, in_root ? NULL : &parent);
+    gtk_tree_store_set (mm_store, &cat_iter, 0,
+			gtk_entry_get_text (GTK_ENTRY(cat_entry)), -1);
+  }
 }
 
 static void tree_changed_cb (GtkTreeModel *treemodel,
 			     GtkTreePath *arg1,
 			     gpointer user_data) {
-  update_menu (media_menu);
+  update_menu (media_menu[0]);
+  update_menu (media_menu[1]);
 }
 
-static int load_mediamarks (gchar *fname) {
-
-  char                 *mmfile;
-
-  mmfile = read_entire_file_ascii (fname);
+static int load_mediamarks (gchar *fname)
+{
+  char *mmfile = read_entire_file (fname, NULL);
 
   if (mmfile) {
     xml_node_t *node;
@@ -543,177 +467,48 @@ void mm_import (void) {
 
   gchar *fname;
 
-  fname = modal_file_dialog ("Select mediamarks file to load...", NULL);
+  fname = modal_file_dialog (_("Select media marks file to load..."), NULL);
 
   if (fname) {
 
-    if (!load_mediamarks (fname))
-      display_error ("Mediamarks load failed",
-		     "Couldn't load mediamarks file"
-		     "%s", fname);
+    if (!load_mediamarks (fname) && errno != ENOENT)
+      display_error (_("Media marks load failed"),
+		     _("Couldn't load media marks file %s\n%s"),
+		     fname, strerror (errno));
   }
 }
 
-void mediamarks_init (void) {
-
-  GtkWidget            *tree, *b, *scrolled_window, *hbox;
-  GtkWidget            *button, *label;
-  gchar                *fname;
-  GtkCellRenderer      *cell;
-  GtkTreeViewColumn    *column;
-   
-  /*
-   * init tree store
-   */
- 
-  mm_store = gtk_tree_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
-
-  /* 
-   * now parse the mediamarks file
-   */
-
-  fname = g_strconcat(g_get_home_dir(), "/.gxine/mediamarks", NULL);
-
-  if (!load_mediamarks (fname)) {
-    /* load sample mediamarks */
-
-    g_free (fname);
-
-    fname = g_strconcat (GXINE_MISCDIR, "/mediamarks", NULL);
-    load_mediamarks (fname);
-  }
-
-  g_free(fname);
-
-  g_signal_connect (G_OBJECT (mm_store), "row-changed",
-		    G_CALLBACK (tree_changed_cb), NULL);
-  g_signal_connect (G_OBJECT (mm_store), "row-deleted",
-		    G_CALLBACK (tree_changed_cb), NULL);
-  g_signal_connect (G_OBJECT (mm_store), "row-has-child-toggled",
-		    G_CALLBACK (tree_changed_cb), NULL);
-  g_signal_connect (G_OBJECT (mm_store), "row-inserted",
-		    G_CALLBACK (tree_changed_cb), NULL);
-  g_signal_connect (G_OBJECT (mm_store), "rows-reordered",
-		    G_CALLBACK (tree_changed_cb), NULL);
-
-  update_menu (media_menu);
-
-
-  /*
-   * create (for now invisible) mm_manage dialog
-   */
-
-  manage_dlg = gtk_dialog_new ();
-  gtk_window_set_title (GTK_WINDOW (manage_dlg), "Manage Mediamarks...");
-  gtk_window_set_default_size (GTK_WINDOW (manage_dlg), 400, 250);
-  g_signal_connect( GTK_OBJECT (manage_dlg), "delete_event",
-		      G_CALLBACK (win_delete_cb), NULL );
-  b = gtk_dialog_add_button (GTK_DIALOG (manage_dlg), GTK_STOCK_CLOSE, 1);
-  g_signal_connect (GTK_OBJECT(b), "clicked",
-		    G_CALLBACK (close_cb),
-		    manage_dlg);
-
-  /* add a nice tree view widget here */
-
-  tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL(mm_store));  
-  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree), TRUE);
-
-  cell = gtk_cell_renderer_text_new ();
-  column = gtk_tree_view_column_new_with_attributes ("Mediamarks",
-						     cell,
-						     "text", 0,
-						     NULL);
-  gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
-			       GTK_TREE_VIEW_COLUMN (column));
-
-  column = gtk_tree_view_column_new_with_attributes ("mrl",
-						     cell,
-						     "text", 1,
-						     NULL);
-  gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
-			       GTK_TREE_VIEW_COLUMN (column));
-  
-  gtk_tree_view_set_reorderable (GTK_TREE_VIEW (tree), TRUE);
-
-  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
-				  GTK_POLICY_AUTOMATIC, 
-				  GTK_POLICY_AUTOMATIC);
-  gtk_container_add (GTK_CONTAINER (scrolled_window), tree);
-
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(manage_dlg)->vbox), scrolled_window,
-		      TRUE, TRUE, 2);
-
-  /* add edit button bar */
-  hbox = gtk_hbox_new (2, 2);
-
-  button = gtk_button_new_with_label ("New Category...");
-  g_signal_connect (button, "clicked", G_CALLBACK(new_category_cb), tree);
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 2);  
-
-  button = gtk_button_new_with_label ("Edit...");
-  g_signal_connect (button, "clicked", G_CALLBACK(edit_cb), tree);
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 2);  
-
-  button = gtk_button_new_with_label ("Delete");
-  g_signal_connect (button, "clicked", G_CALLBACK(delete_cb), tree);
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 2);  
-
-  gtk_box_pack_end (GTK_BOX(GTK_DIALOG(manage_dlg)->vbox), hbox,
-		    FALSE, FALSE, 2);
-
-  is_visible = FALSE;
-
-  /*
-   * create (for now invisible) category name entry dialog
-   */
-
-  cat_dlg = gtk_dialog_new ();
-  gtk_window_set_title (GTK_WINDOW (cat_dlg), "Enter category name...");
-  g_signal_connect( GTK_OBJECT (cat_dlg), "delete_event",
-		      G_CALLBACK (cat_delete_cb), NULL );
-
-  b = gtk_dialog_add_button (GTK_DIALOG (cat_dlg), GTK_STOCK_CLOSE, 1);
-  g_signal_connect (GTK_OBJECT(b), "clicked",
-		      G_CALLBACK (cat_close_cb),
-		      cat_dlg);
-
-  /* all we have is a simple text entry */
-
-  hbox = gtk_hbox_new (2, 2);
-
-  label = gtk_label_new ("Category name:");
-
-  gtk_box_pack_start (GTK_BOX (hbox), label,
-		      FALSE, FALSE, 0);
-  
-  cat_entry = gtk_entry_new (); 
-  gtk_signal_connect (GTK_OBJECT(cat_entry), "activate",
-		      GTK_SIGNAL_FUNC (cat_close_cb),
-		      cat_dlg);  
-  gtk_box_pack_start (GTK_BOX (hbox), cat_entry,
-		      TRUE, TRUE, 0);
-
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (cat_dlg)->vbox), hbox,
-		      TRUE, TRUE, 0);
-}
-
 void mm_add_show () {
 
   play_item_t *play_item;
   GtkTreeIter  iter;
   int          pos, pos_time, len;
+  char	      *dot;
+  const char  *title = xine_get_meta_info (stream, XINE_META_INFO_TITLE);
 
   play_item = play_item_copy (playlist_get_item (playlist_get_list_pos()));
 
-  gtk_xine_get_pos_length (GTK_XINE (gtx), 
-			   &pos, &pos_time, &len);
+  /* default title? try to get title from meta-info */
+  dot = strrchr (play_item->mrl, '.');
+  if (!dot)
+    dot = play_item->mrl + strlen (play_item->mrl);
+  len = strlen (play_item->title);
+  if (title && dot - len > play_item->mrl &&
+      !strncmp (dot - len, play_item->title, len))
+  {
+    free (play_item->title);
+    play_item->title = strdup (title);
+  }
 
-  play_item->start_time = pos_time;
-  play_item_edit (play_item);
-  gtk_tree_store_append (mm_store, &iter, NULL);
-  gtk_tree_store_set (mm_store, &iter, 0, play_item->title, 
-		      1, play_item->mrl, 2, play_item, -1);
+  xine_get_pos_length (stream, &pos, &pos_time, &len);
+  play_item->start_time = (len >= 0) ? pos_time : 0;
+
+  if (play_item_edit (play_item, PLAY_ITEM_MEDIAMARK_NEW, title))
+  {
+    gtk_tree_store_append (mm_store, &iter, NULL);
+    gtk_tree_store_set (mm_store, &iter, 0, play_item->title,
+			1, play_item->mrl, 2, play_item, -1);
+  }
 }
 
 
@@ -722,10 +517,10 @@ void mm_manage_show () {
   if (is_visible) {
     is_visible = FALSE;
     gtk_widget_hide (manage_dlg);
+    mm_save();
   } else {
     is_visible = TRUE;
     gtk_widget_show_all (manage_dlg);
-    gtk_widget_map (manage_dlg);
   }
 }
 
@@ -751,7 +546,7 @@ static void save_items (FILE *f, int depth, GtkTreeIter *iter) {
 			      iter, 0, &v);
     id = g_value_peek_pointer (&v);
 
-    if (id && strcmp (id, PLACEHOLDER)) {
+    if (id) {
 
       GValue vi;
 
@@ -807,9 +602,228 @@ void mm_save (void) {
 
     fprintf (f, "</GXINEMM>\n");
 
-    fclose (f);
+    if (ferror (f))
+      printf (_("mediamarks: save to '%s' failed: %s\n"),
+	      fname, strerror (errno));
+    if (fclose (f))
+      printf (_("mediamarks: save to '%s' failed: %s\n"),
+	      fname, strerror (errno));
   } else
-    printf ("mediamarks: save to '%s' failed!\n", fname);
+    printf (_("mediamarks: save to '%s' failed: %s\n"),
+	    fname, strerror (errno));
 
   g_free(fname);
 }
+
+static JSBool js_mm_add_show (JSContext *cx, JSObject *obj, uintN argc, 
+			      jsval *argv, jsval *rval) {
+
+  /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
+
+  se_log_fncall ("mm_add_show");
+
+  mm_add_show ();
+  
+  return JS_TRUE;
+}
+
+static JSBool js_mm_manage_show (JSContext *cx, JSObject *obj, uintN argc, 
+				 jsval *argv, jsval *rval) {
+
+  /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
+
+  se_log_fncall ("mm_manage_show");
+
+  mm_manage_show ();
+  
+  return JS_TRUE;
+}
+
+static JSBool js_import_mediamarks (JSContext *cx, JSObject *obj, uintN argc, 
+				    jsval *argv, jsval *rval) {
+
+  /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
+
+  se_log_fncall ("import_mediamarks");
+
+  mm_import ();
+  
+  return JS_TRUE;
+}
+
+static void mm_response_cb (GtkDialog *dbox, int response, gpointer data)
+{
+  switch (response)
+  {
+  case GTK_RESPONSE_ACCEPT:
+    mm_save ();
+    break;
+  default:
+    is_visible = FALSE;
+    gtk_widget_hide (manage_dlg);
+  }
+}
+
+void mediamarks_init (void) {
+
+  GtkWidget            *tree, *b, *scrolled_window, *hbox, *label;
+  gchar                *fname;
+  GtkCellRenderer      *cell;
+  GtkTreeViewColumn    *column;
+   
+  /*
+   * init tree store
+   */
+ 
+  mm_store = gtk_tree_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
+
+  /* 
+   * now parse the mediamarks file
+   */
+
+  fname = g_strconcat(g_get_home_dir(), "/.gxine/mediamarks", NULL);
+
+  if (!load_mediamarks (fname)) {
+    /* load sample mediamarks */
+
+    g_free (fname);
+
+    fname = g_strconcat (miscdir, "/mediamarks", NULL);
+    load_mediamarks (fname);
+  }
+
+  g_free(fname);
+
+  g_signal_connect (G_OBJECT (mm_store), "row-changed",
+		    G_CALLBACK (tree_changed_cb), NULL);
+  g_signal_connect (G_OBJECT (mm_store), "row-deleted",
+		    G_CALLBACK (tree_changed_cb), NULL);
+  g_signal_connect (G_OBJECT (mm_store), "row-has-child-toggled",
+		    G_CALLBACK (tree_changed_cb), NULL);
+  g_signal_connect (G_OBJECT (mm_store), "row-inserted",
+		    G_CALLBACK (tree_changed_cb), NULL);
+  g_signal_connect (G_OBJECT (mm_store), "rows-reordered",
+		    G_CALLBACK (tree_changed_cb), NULL);
+
+  update_menu (media_menu[0]);
+  update_menu (media_menu[1]);
+
+  /*
+   * create (for now invisible) mm_manage dialog
+   */
+
+  manage_dlg = gtk_dialog_new_with_buttons (_("Manage media marks..."), NULL, 0,
+				GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+				GTK_STOCK_CLOSE, GTK_RESPONSE_DELETE_EVENT,
+				NULL);
+  gtk_window_set_default_size (GTK_WINDOW (manage_dlg), 400, 250);
+  g_signal_connect (GTK_OBJECT(manage_dlg), "response",
+		    G_CALLBACK(mm_response_cb), NULL);
+  hide_on_delete (manage_dlg, &is_visible);
+
+  /* add a nice tree view widget here */
+
+  mm_tree = tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL(mm_store));
+  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree), TRUE);
+
+  cell = gtk_cell_renderer_text_new ();
+  column = gtk_tree_view_column_new_with_attributes (_("Media mark"), cell,
+						     "text", 0, NULL);
+  gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
+			       GTK_TREE_VIEW_COLUMN (column));
+
+  column = gtk_tree_view_column_new_with_attributes ("MRL", cell,
+						     "text", 1, NULL);
+  gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
+			       GTK_TREE_VIEW_COLUMN (column));
+  
+  gtk_tree_view_set_reorderable (GTK_TREE_VIEW (tree), TRUE);
+
+  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+				  GTK_POLICY_AUTOMATIC, 
+				  GTK_POLICY_AUTOMATIC);
+  gtk_container_add (GTK_CONTAINER (scrolled_window), tree);
+
+  g_signal_connect (GTK_OBJECT (tree), "row-activated",
+		    G_CALLBACK (edit_cb), NULL);
+
+  /* add edit button bar */
+  hbox = gtk_hbox_new (FALSE, 2);
+
+  b = gtk_button_new_with_mnemonic (_("_New media mark"));
+  gtk_box_pack_start (GTK_BOX(hbox), b, FALSE, FALSE, 0);
+  g_signal_connect (GTK_OBJECT(b), "clicked",
+		    G_CALLBACK(mm_add_show), NULL);
+
+  b = gtk_button_new_with_mnemonic (_("New _category"));
+  gtk_box_pack_start (GTK_BOX(hbox), b, FALSE, FALSE, 0);
+  g_signal_connect (GTK_OBJECT(b), "clicked",
+		    G_CALLBACK(new_category_cb), tree);
+
+  b = gtk_button_new_with_mnemonic (_("_Edit"));
+  gtk_box_pack_start (GTK_BOX(hbox), b, FALSE, FALSE, 0);
+  g_signal_connect (GTK_OBJECT(b), "clicked",
+		    G_CALLBACK(edit_cb), NULL);
+
+  b = gtk_button_new_from_stock (GTK_STOCK_DELETE);
+  gtk_box_pack_start (GTK_BOX(hbox), b, FALSE, FALSE, 0);
+  g_signal_connect (GTK_OBJECT(b), "clicked",
+		    G_CALLBACK(delete_cb), NULL);
+
+  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(manage_dlg)->vbox), hbox,
+		    FALSE, FALSE, 2);
+
+  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(manage_dlg)->vbox), scrolled_window,
+		      TRUE, TRUE, 2);
+
+  is_visible = FALSE;
+
+  /*
+   * create (for now invisible) category name entry dialog
+   */
+
+  cat_dlg = gtk_dialog_new_with_buttons (_("Enter category name..."), NULL, 0,
+				GTK_STOCK_CANCEL, GTK_RESPONSE_DELETE_EVENT,
+				GTK_STOCK_OK, GTK_RESPONSE_OK,
+				NULL);
+  hide_on_delete (cat_dlg, NULL);
+  g_signal_connect (GTK_OBJECT (cat_dlg), "response",
+		    G_CALLBACK (cat_response_cb), NULL);
+  gtk_dialog_set_default_response (GTK_DIALOG(cat_dlg), GTK_RESPONSE_OK);
+
+  /* all we have is a simple text entry */
+
+  hbox = gtk_hbox_new (2, 2);
+
+  label = gtk_label_new (_("Category name:"));
+
+  gtk_box_pack_start (GTK_BOX (hbox), label,
+		      FALSE, FALSE, 0);
+
+  cat_entry = gtk_entry_new ();
+/*
+  gtk_signal_connect (GTK_OBJECT(cat_entry), "activate",
+		      GTK_SIGNAL_FUNC (cat_close_cb),
+		      cat_dlg);
+*/
+  gtk_box_pack_start_defaults (GTK_BOX (hbox), cat_entry);
+
+  gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (cat_dlg)->vbox), hbox);
+
+  /* script engine functions */
+
+  {
+    static const se_f_def_t defs[] = {
+      { "mm_add_show", js_mm_add_show, 0, 0,
+	SE_GROUP_DIALOGUE, NULL, NULL },
+      { "mm_manage_show", js_mm_manage_show, 0, 0,
+	SE_GROUP_DIALOGUE, NULL, NULL },
+      { "import_mediamarks", js_import_mediamarks, 0, 0,
+	SE_GROUP_FILE, NULL, NULL },
+      { NULL }
+    };
+    se_defuns (gse, NULL, defs);
+  }
+}
+
