@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: menu.h,v 1.1 2003/02/01 23:30:07 guenter Exp $
+ * $Id: menu.h,v 1.3 2004/11/03 19:47:35 dsalt Exp $
  *
  * main/context menu creation / handling
  */
@@ -25,6 +25,11 @@
 
 #include <gtk/gtk.h>
 
-GtkWidget *create_menus (GtkWidget *window) ;
+#define num_menu_items(tree) (sizeof ((tree)) / sizeof ((tree)[0]))
+
+GtkWidget *create_menu_tree (GtkItemFactory **, GtkWidget *window,
+			     GtkAccelGroup **, GtkItemFactoryEntry *,
+			     gint items);
+void create_menus (GtkWidget *window);
 
 #endif
