@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: noskin_window.h,v 1.1 2003/02/01 23:30:07 guenter Exp $
+ * $Id: noskin_window.h,v 1.2 2004/04/09 18:07:52 guenter Exp $
  *
  * standard, non-skinned main window
  */
@@ -25,8 +25,8 @@
 
 #include <glib.h>
 
-void noskin_main_init (gchar *audio_driver_id, gchar *video_driver_id) ;
+void noskin_main_init (void) ;
 
-void noskin_main_close ();
+void noskin_main_close (void);
 
 #endif
